@@ -1,5 +1,4 @@
 import autograd.numpy as np
-from copy import copy
 
 
 def pass_through(x):
@@ -26,20 +25,20 @@ def bounds_convert(x, bounds):
     funcs = []
     inv_f = []
     for i, (lower, upper) in enumerate(bounds):
-        def add_to_funcs(l, u, i):
-            if (l is None) and (u is None):
+        def add_to_funcs(low, upp, i):
+            if (low is None) and (upp is None):
                 funcs.append(lambda x: pass_through(x))
                 inv_f.append(lambda x: pass_through(x))
-            elif (l == 0) and (u == 1):
+            elif (low == 0) and (upp == 1):
                 D = 10
-                funcs.append(lambda x: D * np.arctanh((2 * x)-1))
-                inv_f.append(lambda x: (np.tanh(x/D) + 1)/2)
-            elif (u is None):
-                funcs.append(lambda x: (inv_adj_relu(x - l)))
-                inv_f.append(lambda x: (adj_relu(x) + l))
-            elif (l is None):
-                funcs.append(lambda x: inv_rev_adj_relu(x - np.copy(u)))
-                inv_f.append(lambda x: np.copy(u) + rev_adj_relu(x))
+                funcs.append(lambda x: D * np.arctanh((2 * x) - 1))
+                inv_f.append(lambda x: (np.tanh(x / D) + 1) / 2)
+            elif (upp is None):
+                funcs.append(lambda x: (inv_adj_relu(x - low)))
+                inv_f.append(lambda x: (adj_relu(x) + low))
+            elif (low is None):
+                funcs.append(lambda x: inv_rev_adj_relu(x - np.copy(upp)))
+                inv_f.append(lambda x: np.copy(upp) + rev_adj_relu(x))
             else:
                 funcs.append(lambda x: pass_through(x))
                 inv_f.append(lambda x: pass_through(x))
@@ -54,23 +53,20 @@ def bounds_convert(x, bounds):
     return transform, inv_trans, funcs, inv_f
 
 
-# Functions to support fixing parameters
-def fix_idx_and_function(dist, fixed, param_map, offset_index_inc, funcs):
+def fix_idx_and_function(fixed, param_map, funcs):
+    n_params = len(param_map)
     if fixed is not None:
         """
         Record to the model that parameters were fixed
         """
-        fixed_idx = [param_map[x] + offset_index_inc for x in fixed.keys()]
-        not_fixed = [x for x in
-                     range(dist.k + offset_index_inc) if x not in fixed_idx]
+        fixed_idx = [param_map[x] for x in fixed.keys()]
+        not_fixed = [x for x in range(n_params) if x not in fixed_idx]
         not_fixed = np.array(not_fixed)
 
         def constraints(p):
-            params = [0] * (dist.k + offset_index_inc)
-            # params = np.empty(self.k + offset_index_inc)
+            params = [0] * (n_params)
             for k, v in fixed.items():
-                params[param_map[k] + offset_index_inc] = (
-                    funcs[param_map[k] + offset_index_inc](v))
+                params[param_map[k]] = funcs[param_map[k]](v)
             for i, v in zip(not_fixed, p):
                 params[i] = v
             return np.array(params)
@@ -82,6 +78,6 @@ def fix_idx_and_function(dist, fixed, param_map, offset_index_inc, funcs):
             return x
 
         fixed_idx = []
-        not_fixed = np.array([x for x in range(dist.k + offset_index_inc)])
+        not_fixed = np.array([x for x in range(n_params)])
 
     return const, fixed_idx, not_fixed
